// Vector.h: interface for the Vector class.
//
//////////////////////////////////////////////////////////////////////

#ifndef VEC_3F_H
#define VEC_3F_H

//#if _MSC_VER > 1000
//#pragma once
//#endif // _MSC_VER > 1000
#include <math.h>
class Vec3f 
{
public:
	float coords[3];
	//   float x;
   // float y;
   // float z;
	float* getCoords() {return coords;}  //const
	const  Vec3f & operator=(const  Vec3f & rhs)
	{
		if (this != &rhs)
		{
		for (int i = 0; i < 3;i++)
		{
			coords[i] = rhs.coords[i];
		}
		}
		return *this;
	}
	float& operator[](int i) {return coords[i];} // const
		Vec3f( const float * cs)
        { coords[0] = cs[0];  coords[1] = cs[1]; coords[2]= cs[2]; }
		Vec3f( const float _x = 0.0f, 
           const float _y = 0.0f, 
           const float _z = 0.0f )
        { coords[0] = _x;  coords[1] = _y;   coords[2] = _z; }
	Vec3f( const Vec3f &vec )  
        { coords[0] = vec.coords[0];  coords[1]= vec.coords[1];  coords[2] = vec.coords[2]; }
	
	virtual ~Vec3f();
	 // Vector Addition
    Vec3f operator+( const Vec3f & vec ) const  // returns this + vec
        { return Vec3f(coords[0]+vec.coords[0], coords[1]+vec.coords[1], coords[2]+vec.coords[2]); } 
    Vec3f &operator+=( const Vec3f & vec )      // this += vec
        { coords[0]+=vec.coords[0]; coords[1]+=vec.coords[1]; coords[2]+=vec.coords[2];
          return *this; } 

    // Vector Subtraction
    Vec3f operator-() const                     // returns -this
        { return Vec3f(-coords[0], -coords[1], -coords[2]); } 
    Vec3f operator-( const Vec3f & vec ) const  // returns this - vec
        { return Vec3f(coords[0]-vec.coords[0], coords[1]-vec.coords[1], coords[2]-vec.coords[2]); } 
    Vec3f &operator-=( const Vec3f & vec )      // this -= vec
        { coords[0]-=vec.coords[0]; coords[1]-=vec.coords[1]; coords[2]-=vec.coords[2];
	      return *this; } 

    // Scaling Multiplication
    Vec3f operator*( const float f ) const      // returns this * f
        { return Vec3f(coords[0]*f, coords[1]*f, coords[2]*f); } 
    friend Vec3f operator*( const float f, const Vec3f &vec )  // returns f * this
        { return Vec3f(vec.coords[0]*f, vec.coords[1]*f, vec.coords[2]*f); } 
    Vec3f &operator*=( const float f )          // this *= f
        { coords[0]*=f; coords[1]*=f; coords[2]*=f;
          return *this; } 

    // Scaling Division
    Vec3f operator/( const float f ) const      // returns this / f
        { return Vec3f(coords[0]/f, coords[1]/f, coords[2]/f); } 
    Vec3f &operator/=( const float f )          // this /= f
        { coords[0]/=f; coords[1]/=f; coords[2]/=f;
          return *this; } 

    // Vector Multiplication
    Vec3f operator*( const Vec3f & vec ) const  // !WARNING (MEMBERWISE): returns this.? * vec.?
        { return Vec3f(coords[0]*vec.coords[0], coords[1]*vec.coords[1], coords[2]*vec.coords[2]); } 
    Vec3f &operator*=( const Vec3f & vec )      // !WARNING (MEMBERWISE): this.? *= vec.?
        { coords[0]*=vec.coords[0]; coords[1]*=vec.coords[1]; coords[2]*=vec.coords[2];
          return *this; } 

    // Vector Division
    Vec3f operator/( const Vec3f & vec ) const  // !WARNING (MEMBERWISE): returns this.? / vec.?
        { return Vec3f(coords[0]/vec.coords[0], coords[1]/vec.coords[1], coords[2]/vec.coords[2]); } 
    Vec3f &operator/=( const Vec3f & vec )      // !WARNING (MEMBERWISE): this.? /= vec.?
        { coords[0]/=vec.coords[0]; coords[1]/=vec.coords[1]; coords[2]/=vec.coords[2];
          return *this; } 

    // Dot Product
    float dot( const Vec3f & vec ) const        // returns dot value of this and vec
        { return coords[0]*vec.coords[0] + coords[1]*vec.coords[1] + coords[2]*vec.coords[2]; } 
    
    // Cross Product
    Vec3f cross( const Vec3f & vec ) const      // returns cross product of this and vec
        { return Vec3f(coords[1]*vec.coords[2] - coords[2]*vec.coords[1], 
                       coords[2]*vec.coords[0] - coords[0]*vec.coords[2], 
                       coords[0]*vec.coords[1]- coords[1]*vec.coords[0]); } 
    Vec3f &cross_equals( const Vec3f & vec )    // this = this cross vec
        { float _x = coords[1]*vec.coords[2] - coords[2]*vec.coords[1];
          float _y = coords[2]*vec.coords[0] - coords[0]*vec.coords[2]; 
          float _z = coords[0]*vec.coords[1] - coords[1]*vec.coords[0];
          coords[0] = _x;  coords[1] = _y;  coords[2] = _z; 
          return *this; } 

    // Length
    float Length() const                // returns length
        { return (float)sqrt(coords[0]*coords[0] + coords[1]*coords[1] + coords[2]*coords[2]); } 
    float length_square() const         // returns length squared
        { return (coords[0]*coords[0] + coords[1]*coords[1] + coords[2]*coords[2]); } 
    
    // Normalize
    Vec3f &Normalize()                  // normalizes this
        { float f=(float)sqrt(coords[0]*coords[0] + coords[1]*coords[1] + coords[2]*coords[2]);
	if (f > .00001)
	{
		coords[0]/=f; coords[1]/=f; coords[2]/=f;}
          return *this; } 
    Vec3f norm_value() const            // returns normalized value, does not modify this
        { float f=Length();
          return Vec3f(coords[0]/f, coords[1]/f, coords[2]/f); } 


};

#endif // !defined(AFX_VECTOR_H__C80EF4CA_9FBE_41E3_B109_F13F0DCC980C__INCLUDED_)
